'use strict';

var dir = require('node-dir');
// https://github.com/fshost/node-dir


///////////////////////////////////////// FNS /////////////////

function conditionChecker(arr) {
  if (arr.constructor != Array) {
    throw new Error('Conditions should be passed in form of array of regexs');
  }
  arr.forEach(function (cond) {
    if (cond.constructor != RegExp) {
      throw new Error('Invalid (not RegExp) condition passed!');
    }
  });
  return arr;
}

function includeFn(res) {
  if (this.include === undefined) return res; // skip condition
  var incFilters = conditionChecker(this.include);
  function reducer(accumulator, current) {
    return accumulator.filter(function (val) {
      var test = current.test(val) === true;
      return test;
    });
  }
  var filtredRes = incFilters.reduce(reducer, res);
  // console.log('filtredRes', filtredRes);
  return filtredRes;
}

function excludeFn(res) {
  if (this.exclude === undefined) return res; // skip condition
  var exclFilters = conditionChecker(this.exclude);
  function reducer(accumulator, current) {
    return accumulator.filter(function (val) {
      var test = current.test(val) === false;
      return test;
    });
  }
  var filtredRes = exclFilters.reduce(reducer, res);
  // console.log('filtredRes', filtredRes);
  return filtredRes;
}

///////////////////////////////////////// MAIN /////////////////
// dla wersji 0.16 bez promise / recznie przekopoiowany kod z 17-stki
function getFilesList() {
  var dirPath = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : process.cwd();
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  // code of node-dir
  function promiseFiles(dirPath, type, options) {
    type = type || 'file';

    var processor = function processor(res, rej) {
      var cb = function cb(err, data) {
        if (err) return rej(err);
        res(data);
      };

      dir.files(dirPath, type, cb, options);
    };
    return new Promise(processor);
  }

  return promiseFiles(dirPath).then(includeFn.bind(options)).then(excludeFn.bind(options)).catch(function (err) {
    // console.log('err', err);
    return Error('q-mod-myfs - getFilesList():  ' + err);
  });
}

module.exports = {
  getFilesList: getFilesList
};

// wersja 0.17 juz ma promise
// function getFilesList(dirPath = process.cwd()) {
//   return dir.promiseFiles(dirPath)
//     .then(files => {
//       return files;
//     });
// }