'use strict';

var archiver = require('archiver');
// https://github.com/archiverjs/node-archiver
var filenamify = require('filenamify');
// https://github.com/sindresorhus/filenamify

var path = require('path');
var fs = require('fs');

var _require = require('q-mod-date');

var newDate = _require.newDate;

var ifExists = require('./q-mod-myfs-ifexists');

var fns = {
  ///////////////////////////////////////// HELPERS /////////////////
  getArchivedDirNameFromCWD: function getArchivedDirNameFromCWD() {
    var output = path.basename(this.vars.cwd).split(path.sep).pop();
    if (output === '') output = 'thisRoot';
    this.vars.cwdDir = output;
  },
  generateArchiveFileName: function generateArchiveFileName() {
    var zN = this.args.zipName;
    // const safeZipName = this.args.zipName && filenamify(zN); // niepotrzebne ?
    var safeWhatToZip = this.args.whatToZip && filenamify(this.args.whatToZip, { replacement: '-' });
    if (!zN) {
      if (this.args.whatToZip === '') {
        this.vars.zipFileName = this.vars.cwdDir + '-' + newDate('long') + '.zip';
        this.vars.zipFileNameFullPath = path.resolve(this.vars.cwd, this.vars.zipFileName);
      } else {
        // whatToZip is passed
        this.vars.zipFileName = safeWhatToZip + '-' + newDate('long') + '.zip';
        this.vars.zipFileNameFullPath = path.resolve(this.vars.cwd, this.vars.zipFileName);
      }
    } else {
      // zipName not passed
      var strippedName = path.basename(zN); // pierwotnie: safeZipName
      var safeStrippedName = filenamify(strippedName, { replacement: '-' });
      this.vars.zipFileName = safeStrippedName + '.zip';
      this.vars.zipFileNameFullPath = path.resolve(this.vars.cwd, zN + '.zip'); //! musi być zN
    }
  },


  ///////////////////////////////////////// CORE FNS /////////////////
  generateSourcePathToZip: function generateSourcePathToZip() {
    this.vars.dirToZip = path.resolve(this.vars.cwd, this.args.whatToZip);
  },
  generateOutputPathToSaveZip: function generateOutputPathToSaveZip() {
    this.vars.pathToCreateZipFile = this.vars.zipFileNameFullPath;
  },
  coputeWhatToIncludeToArch: function coputeWhatToIncludeToArch() {
    var _this = this;

    if (this.options.archiverIncludeDotDirs) this.logs.push('include: dot dirs');
    var glob = this.vars.archiverGlob.slice();
    // exlude zipFile (e.i. file.zip) in archive file
    glob.push('!' + this.vars.zipFileName);
    this.logs.push('exlude: !' + this.vars.zipFileName);
    this.options.exclusions.forEach(function (item) {
      // pattern: ['**/*','!node_modules{,/**}']
      // uwaga: appending also files (!some.json{,/**}), but so far it works
      glob.push('!' + item + '{,/**}');
      _this.logs.push('exlude: ' + item);
    });
    this.vars.whatToIncludeToArch = glob;
  },
  executePromisForZip: function executePromisForZip(resolve, reject) {
    var _this2 = this;

    var zipper = archiver('zip');
    zipper.bulk([{
      expand: this.options.archiverExpand,
      dot: this.options.archiverIncludeDotDirs,
      cwd: this.vars.dirToZip,
      src: this.vars.whatToIncludeToArch
    }]);
    if (ifExists(this.vars.pathToCreateZipFile, 'boolean')) this.logs.push('warning: overwrite existing zip file!');
    this.temp.outputStream = fs.createWriteStream(this.vars.pathToCreateZipFile);
    zipper.pipe(this.temp.outputStream);
    this.temp.outputStream.on('close', function () {
      // ratuje mnie z właściwym contextem,
      // w przypadku cb function(){} trzeba by było bindować :)
      _this2.vars.size = zipper.pointer();
      resolve(_this2);
    });

    zipper.on('end', function () {
      _this2.logs.push('Archive completed successfully.');
    });

    zipper.on('error', function (err) {
      zipper.abort(); //not always useful but might save trouble
      reject('q-mod-myfs - zipDir() - Unable to archive: ' + err);
    });

    zipper.finalize();
  },
  performZip: function performZip() {
    // dryRun / hotRun fork
    if (!this.options.dryRun) {
      this.logs.push('Performing real run of script.');
      return new Promise(this.executePromisForZip.bind(this));
    } else {
      this.logs.push('Performing dry run of script.');
      this.logs.push('Dryrun of script completed successfully.');
      return Promise.resolve(this);
    }
  }
};

///////////////////////////////////////// STATE /////////////////
function createState() {
  var composeStateObj = Object.assign(Object.create(fns), {
    // the hard way: in every instance new objs:
    logs: [],
    args: {},
    options: {
      dryRun: false,
      exclusions: [],
      archiverExpand: true,
      archiverIncludeDotDirs: false
    },
    vars: {
      cwd: process.cwd(),
      archiverGlob: ['**/*']
    },
    temp: {
      outputStram: undefined
    }
  });
  return composeStateObj;
}

function margeOptions(options, passedOptions) {
  options = Object.assign(options, passedOptions);
}

///////////////////////////////////////// MAIN /////////////////
function zipDir() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var zipName = _ref.zipName;
  var _ref$whatToZip = _ref.whatToZip;
  var whatToZip = _ref$whatToZip === undefined ? '' : _ref$whatToZip;
  var _ref$options = _ref.options;
  var options = _ref$options === undefined ? {} : _ref$options;

  //////////////////////  state
  var s = createState();
  s.logs.push('Started q-mod-myfs - zipDir() script at:');
  s.logs.push(newDate('long'));
  s.args = { zipName: zipName, whatToZip: whatToZip, options: options };
  margeOptions(s.options, options);

  //////////////////////  generate vars
  s.getArchivedDirNameFromCWD();
  // cwdDir - need for dif zipFile generate
  s.generateArchiveFileName();
  // dif cwdDir name + long date
  s.generateSourcePathToZip();
  // dif cwd
  s.generateOutputPathToSaveZip();
  // dif cwd
  s.coputeWhatToIncludeToArch();
  // dif all dir

  ////////////////////// main fn
  return s.performZip();
}

module.exports = {
  zipDir: zipDir,
  fns: fns
};